@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: BankRomCheat.cxx 2199 2011-01-01 16:04:32Z stephena $
+// $Id: BankRomCheat.cxx 2360 2012-01-20 23:56:49Z stephena $
 //============================================================================
 
 #include "Console.hxx"
@@ -33,6 +33,10 @@ BankRomCheat::BankRomCheat(OSystem* os, const string& name, const string& code)
   address = 0xf000 + unhex(myCode.substr(2, 3));
   value = unhex(myCode.substr(5, 2));
   count = unhex(myCode.substr(7, 1)) + 1;
+
+  // Back up original data; we need this if the cheat is ever disabled
+  for(int i = 0; i < count; ++i)
+    savedRom[i] = myOSystem->console().cartridge().peek(address + i);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -52,7 +56,8 @@ bool BankRomCheat::disable()
 {
   int oldBank = myOSystem->console().cartridge().bank();
   myOSystem->console().cartridge().bank(bank);
-  for(int i=0; i<count; i++)
+
+  for(int i = 0; i < count; ++i)
 		myOSystem->console().cartridge().patch(address + i, savedRom[i]);
 
   myOSystem->console().cartridge().bank(oldBank);
@@ -68,11 +73,9 @@ void BankRomCheat::evaluate()
     int oldBank = myOSystem->console().cartridge().bank();
     myOSystem->console().cartridge().bank(bank);
 
-    for(int i=0; i<count; i++)
-    {
-      savedRom[i] = myOSystem->console().cartridge().peek(address + i);
+    for(int i = 0; i < count; ++i)
       myOSystem->console().cartridge().patch(address + i, value);
-    }
+
     myOSystem->console().cartridge().bank(oldBank);
 
     myEnabled = true;
