@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CheetahCheat.cxx 2199 2011-01-01 16:04:32Z stephena $
+// $Id: CheetahCheat.cxx 2360 2012-01-20 23:56:49Z stephena $
 //============================================================================
 
 #include "Console.hxx"
@@ -29,6 +29,10 @@ CheetahCheat::CheetahCheat(OSystem* os, const string& name, const string& code)
   address = 0xf000 + unhex(code.substr(0, 3));
   value = unhex(code.substr(3, 2));
   count = unhex(code.substr(5, 1)) + 1;
+
+  // Back up original data; we need this if the cheat is ever disabled
+  for(int i = 0; i < count; ++i)
+    savedRom[i] = myOSystem->console().cartridge().peek(address + i);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -46,7 +50,7 @@ bool CheetahCheat::enable()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CheetahCheat::disable()
 {
-  for(int i=0; i<count; i++)
+  for(int i = 0; i < count; ++i)
     myOSystem->console().cartridge().patch(address + i, savedRom[i]);
 
   return myEnabled = false;
@@ -57,11 +61,8 @@ void CheetahCheat::evaluate()
 {
   if(!myEnabled)
   {
-    for(int i=0; i<count; i++)
-    {
-      savedRom[i] = myOSystem->console().cartridge().peek(address + i);
+    for(int i = 0; i < count; ++i)
       myOSystem->console().cartridge().patch(address + i, value);
-    }
 
     myEnabled = true;
   }
