@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FBSurfaceGL.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: FBSurfaceGL.cxx 2501 2012-05-26 22:18:13Z stephena $
 //============================================================================
 
 #ifdef DISPLAY_OPENGL
@@ -42,16 +42,12 @@ FBSurfaceGL::FBSurfaceGL(FrameBufferGL& buffer, uInt32 width, uInt32 height)
   myTexCoordW = (GLfloat) myImageW / myTexWidth;
   myTexCoordH = (GLfloat) myImageH / myTexHeight;
 
-  // Based on experimentation, the following are the fastest 16-bit
-  // formats for OpenGL (on all platforms)
-  myTexture = SDL_CreateRGBSurface(SDL_SWSURFACE,
-                  myTexWidth, myTexHeight, 16,
-#ifdef HAVE_GL_BGRA
-                  0x00007c00, 0x000003e0, 0x0000001f, 0x00000000);
-#else
-                  0x0000f800, 0x000007c0, 0x0000003e, 0x00000000);
-#endif
-  myPitch = myTexture->pitch >> 1;
+  // Create a surface in the same format as the parent GL class
+  const SDL_PixelFormat& pf = myFB.myPixelFormat;
+  myTexture = SDL_CreateRGBSurface(SDL_SWSURFACE, myTexWidth, myTexHeight,
+                  pf.BitsPerPixel, pf.Rmask, pf.Gmask, pf.Bmask, pf.Amask);
+
+  myPitch = myTexture->pitch / pf.BytesPerPixel;
 
   // Associate the SDL surface with a GL texture object
   updateCoords();
@@ -70,18 +66,18 @@ FBSurfaceGL::~FBSurfaceGL()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FBSurfaceGL::hLine(uInt32 x, uInt32 y, uInt32 x2, uInt32 color)
 {
-  uInt16* buffer = (uInt16*) myTexture->pixels + y * myPitch + x;
+  uInt32* buffer = (uInt32*) myTexture->pixels + y * myPitch + x;
   while(x++ <= x2)
-    *buffer++ = (uInt16) myFB.myDefPalette[color];
+    *buffer++ = (uInt32) myFB.myDefPalette[color];
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FBSurfaceGL::vLine(uInt32 x, uInt32 y, uInt32 y2, uInt32 color)
 {
-  uInt16* buffer = (uInt16*) myTexture->pixels + y * myPitch + x;
+  uInt32* buffer = (uInt32*) myTexture->pixels + y * myPitch + x;
   while(y++ <= y2)
   {
-    *buffer = (uInt16) myFB.myDefPalette[color];
+    *buffer = (uInt32) myFB.myDefPalette[color];
     buffer += myPitch;
   }
 }
@@ -99,10 +95,10 @@ void FBSurfaceGL::fillRect(uInt32 x, uInt32 y, uInt32 w, uInt32 h, uInt32 color)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FBSurfaceGL::drawChar(const GUI::Font* font, uInt8 chr,
+void FBSurfaceGL::drawChar(const GUI::Font& font, uInt8 chr,
                            uInt32 tx, uInt32 ty, uInt32 color)
 {
-  const FontDesc& desc = font->desc();
+  const FontDesc& desc = font.desc();
 
   // If this character is not included in the font, use the default char.
   if(chr < desc.firstchar || chr >= desc.firstchar + desc.size)
@@ -130,7 +126,7 @@ void FBSurfaceGL::drawChar(const GUI::Font* font, uInt8 chr,
   }
 
   const uInt16* tmp = desc.bits + (desc.offset ? desc.offset[chr] : (chr * desc.fbbh));
-  uInt16* buffer = (uInt16*) myTexture->pixels +
+  uInt32* buffer = (uInt32*) myTexture->pixels +
                    (ty + desc.ascent - bby - bbh) * myPitch +
                    tx + bbx;
 
@@ -141,7 +137,7 @@ void FBSurfaceGL::drawChar(const GUI::Font* font, uInt8 chr,
 
     for(int x = 0; x < bbw; x++, mask >>= 1)
       if(ptr & mask)
-        buffer[x] = (uInt16) myFB.myDefPalette[color];
+        buffer[x] = (uInt32) myFB.myDefPalette[color];
 
     buffer += myPitch;
   }
@@ -151,14 +147,14 @@ void FBSurfaceGL::drawChar(const GUI::Font* font, uInt8 chr,
 void FBSurfaceGL::drawBitmap(uInt32* bitmap, uInt32 tx, uInt32 ty,
                              uInt32 color, uInt32 h)
 {
-  uInt16* buffer = (uInt16*) myTexture->pixels + ty * myPitch + tx;
+  uInt32* buffer = (uInt32*) myTexture->pixels + ty * myPitch + tx;
 
   for(uInt32 y = 0; y < h; ++y)
   {
     uInt32 mask = 0xF0000000;
     for(uInt32 x = 0; x < 8; ++x, mask >>= 4)
       if(bitmap[y] & mask)
-        buffer[x] = (uInt16) myFB.myDefPalette[color];
+        buffer[x] = (uInt32) myFB.myDefPalette[color];
 
     buffer += myPitch;
   }
@@ -167,10 +163,10 @@ void FBSurfaceGL::drawBitmap(uInt32* bitmap, uInt32 tx, uInt32 ty,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FBSurfaceGL::drawPixels(uInt32* data, uInt32 tx, uInt32 ty, uInt32 numpixels)
 {
-  uInt16* buffer = (uInt16*) myTexture->pixels + ty * myPitch + tx;
+  uInt32* buffer = (uInt32*) myTexture->pixels + ty * myPitch + tx;
 
   for(uInt32 i = 0; i < numpixels; ++i)
-    *buffer++ = (uInt16) data[i];
+    *buffer++ = (uInt32) data[i];
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -260,13 +256,11 @@ void FBSurfaceGL::update()
     // and antialiasing
     myGL.ActiveTexture(GL_TEXTURE0);
     myGL.BindTexture(GL_TEXTURE_2D, myTexID);
-    myGL.TexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, myTexWidth, myTexHeight,
-#ifdef HAVE_GL_BGRA
-                      GL_BGRA, GL_UNSIGNED_SHORT_1_5_5_5_REV,
-#else
-                      GL_RGBA, GL_UNSIGNED_SHORT_5_5_5_1,
-#endif
-                      myTexture->pixels);
+    myGL.PixelStorei(GL_UNPACK_ALIGNMENT, 1);
+    myGL.PixelStorei(GL_UNPACK_ROW_LENGTH, myPitch);
+    myGL.TexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, myImageW, myImageH,
+                       GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV,
+                       myTexture->pixels);
 
     myGL.EnableClientState(GL_VERTEX_ARRAY);
     myGL.EnableClientState(GL_TEXTURE_COORD_ARRAY);
@@ -321,14 +315,11 @@ void FBSurfaceGL::reload()
   myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 
   // Create the texture in the most optimal format
-  myGL.TexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
-                 myTexWidth, myTexHeight, 0,
-#ifdef HAVE_GL_BGRA
-                 GL_BGRA, GL_UNSIGNED_SHORT_1_5_5_5_REV,
-#else
-                 GL_RGBA, GL_UNSIGNED_SHORT_5_5_5_1,
-#endif
-                 myTexture->pixels);
+  myGL.PixelStorei(GL_UNPACK_ALIGNMENT, 1);
+  myGL.PixelStorei(GL_UNPACK_ROW_LENGTH, myPitch);
+  myGL.TexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, myTexWidth, myTexHeight, 0,
+                  GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV,
+                  myTexture->pixels);
 
   // Cache vertex and texture coordinates using vertex buffer object
   if(myFB.myVBOAvailable)
