@@ -14,51 +14,45 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FBSurfaceTIA.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: FBSurfaceTIA.cxx 2524 2012-06-04 15:52:56Z stephena $
 //============================================================================
 
 #ifdef DISPLAY_OPENGL
 
+#include <cmath>
+
 #include "Font.hxx"
 #include "FrameBufferGL.hxx"
 #include "TIA.hxx"
+#include "NTSCFilter.hxx"
 
 #include "FBSurfaceTIA.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-FBSurfaceTIA::FBSurfaceTIA(FrameBufferGL& buffer, uInt32 baseWidth, uInt32 baseHeight,
-                           uInt32 imgX, uInt32 imgY, uInt32 imgW, uInt32 imgH)
+FBSurfaceTIA::FBSurfaceTIA(FrameBufferGL& buffer)
   : myFB(buffer),
     myGL(myFB.p_gl),
     myTexture(NULL),
     myVBOID(0),
-    myImageX(imgX),
-    myImageY(imgY),
-    myImageW(imgW),
-    myImageH(imgH)
+    myBaseW(0),
+    myBaseH(0),
+    myScanlinesEnabled(false),
+    myScanlineIntensityI(50),
+    myScanlineIntensityF(0.5)
 {
   myTexID[0] = myTexID[1] = 0;
 
-  // Fill buffer struct with valid data
-  myTexWidth  = FrameBufferGL::power_of_two(baseWidth);
-  myTexHeight = FrameBufferGL::power_of_two(baseHeight);
-  myTexCoordW = (GLfloat) baseWidth / myTexWidth;
-  myTexCoordH = (GLfloat) baseHeight / myTexHeight;
+  // Texture width is set to contain all possible sizes for a TIA image,
+  // including Blargg filtering
+  myTexWidth  = FrameBufferGL::power_of_two(ATARI_NTSC_OUT_WIDTH(160));
+  myTexHeight = FrameBufferGL::power_of_two(320);
 
-  // Based on experimentation, the following are the fastest 16-bit
-  // formats for OpenGL (on all platforms)
-  myTexture = SDL_CreateRGBSurface(SDL_SWSURFACE,
-                  myTexWidth, myTexHeight, 16,
-#ifdef HAVE_GL_BGRA
-                  0x00007c00, 0x000003e0, 0x0000001f, 0x00000000);
-#else
-                  0x0000f800, 0x000007c0, 0x0000003e, 0x00000000);
-#endif
-  myPitch = myTexture->pitch >> 1;
+  // Create a surface in the same format as the parent GL class
+  const SDL_PixelFormat& pf = myFB.myPixelFormat;
+  myTexture = SDL_CreateRGBSurface(SDL_SWSURFACE, myTexWidth, myTexHeight,
+                  pf.BitsPerPixel, pf.Rmask, pf.Gmask, pf.Bmask, pf.Amask);
 
-  // Associate the SDL surface with a GL texture object
-  updateCoords();
-  reload();
+  myPitch = myTexture->pitch / pf.BytesPerPixel;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -95,55 +89,71 @@ void FBSurfaceTIA::update()
   uInt8* previousFrame = myTIA->previousFrameBuffer();
   uInt32 width         = myTIA->width();
   uInt32 height        = myTIA->height();
-  uInt16* buffer       = (uInt16*) myTexture->pixels;
+  uInt32* buffer       = (uInt32*) myTexture->pixels;
 
-  if(!myFB.myUsePhosphor)
+  // TODO - Eventually 'phosphor' won't be a separate mode, and will become
+  //        a post-processing filter by blending several frames.
+  switch(myFB.myFilterType)
   {
-    uInt32 bufofsY    = 0;
-    uInt32 screenofsY = 0;
-    for(uInt32 y = 0; y < height; ++y )
+    case FrameBufferGL::kNormal:
     {
-      uInt32 pos = screenofsY;
-      for(uInt32 x = 0; x < width; ++x )
-        buffer[pos++] = (uInt16) myFB.myDefPalette[currentFrame[bufofsY + x]];
-
-      bufofsY    += width;
-      screenofsY += myPitch;
-    }
-  }
-  else
-  {
-    uInt32 bufofsY    = 0;
-    uInt32 screenofsY = 0;
-    for(uInt32 y = 0; y < height; ++y )
-    {
-      uInt32 pos = screenofsY;
-      for(uInt32 x = 0; x < width; ++x )
+      uInt32 bufofsY    = 0;
+      uInt32 screenofsY = 0;
+      for(uInt32 y = 0; y < height; ++y)
       {
-        const uInt32 bufofs = bufofsY + x;
-        buffer[pos++] = (uInt16)
-          myFB.myAvgPalette[currentFrame[bufofs]][previousFrame[bufofs]];
+        uInt32 pos = screenofsY;
+        for(uInt32 x = 0; x < width; ++x)
+          buffer[pos++] = (uInt32) myFB.myDefPalette[currentFrame[bufofsY + x]];
+
+        bufofsY    += width;
+        screenofsY += myPitch;
       }
-      bufofsY    += width;
-      screenofsY += myPitch;
+      break;
+    }
+    case FrameBufferGL::kPhosphor:
+    {
+      uInt32 bufofsY    = 0;
+      uInt32 screenofsY = 0;
+      for(uInt32 y = 0; y < height; ++y)
+      {
+        uInt32 pos = screenofsY;
+        for(uInt32 x = 0; x < width; ++x)
+        {
+          const uInt32 bufofs = bufofsY + x;
+          buffer[pos++] = (uInt32)
+            myFB.myAvgPalette[currentFrame[bufofs]][previousFrame[bufofs]];
+        }
+        bufofsY    += width;
+        screenofsY += myPitch;
+      }
+      break;
+    }
+    case FrameBufferGL::kBlarggNormal:
+    {
+      myFB.myNTSCFilter.blit_single(currentFrame, width, height,
+                                    buffer, myTexture->pitch);
+      break;
+    }
+    case FrameBufferGL::kBlarggPhosphor:
+    {
+      myFB.myNTSCFilter.blit_double(currentFrame, previousFrame, width, height,
+                                    buffer, myTexture->pitch);
+      break;
     }
   }
 
-  // Texturemap complete texture to surface so we have free scaling
-  // and antialiasing
-  myGL.ActiveTexture(GL_TEXTURE0);
-  myGL.BindTexture(GL_TEXTURE_2D, myTexID[0]);
-  myGL.TexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, myTexWidth, myTexHeight,
-#ifdef HAVE_GL_BGRA
-                    GL_BGRA, GL_UNSIGNED_SHORT_1_5_5_5_REV,
-#else
-                    GL_RGBA, GL_UNSIGNED_SHORT_5_5_5_1,
-#endif
-                    myTexture->pixels);
-
   myGL.EnableClientState(GL_VERTEX_ARRAY);
   myGL.EnableClientState(GL_TEXTURE_COORD_ARRAY);
 
+  // Update TIA image (texture 0), then blend scanlines (texture 1)
+  myGL.ActiveTexture(GL_TEXTURE0);
+  myGL.BindTexture(GL_TEXTURE_2D, myTexID[0]);
+  myGL.PixelStorei(GL_UNPACK_ALIGNMENT, 1);
+  myGL.PixelStorei(GL_UNPACK_ROW_LENGTH, myPitch);
+  myGL.TexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, myBaseW, myBaseH,
+                    GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV,
+                    myTexture->pixels);
+
   if(myFB.myVBOAvailable)
   {
     myGL.BindBuffer(GL_ARRAY_BUFFER, myVBOID);
@@ -151,11 +161,10 @@ void FBSurfaceTIA::update()
     myGL.TexCoordPointer(2, GL_FLOAT, 0, (const GLvoid*)(8*sizeof(GLfloat)));
     myGL.DrawArrays(GL_TRIANGLE_STRIP, 0, 4);
 
-#if 0
-    if(1)//myFB.myScanlinesEnabled)
+    if(myScanlinesEnabled)
     {
       myGL.Enable(GL_BLEND);
-      myGL.Color4f(1.0f, 1.0f, 1.0f, 0.5f);
+      myGL.Color4f(1.0f, 1.0f, 1.0f, myScanlineIntensityF);
       myGL.BlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
       myGL.BindTexture(GL_TEXTURE_2D, myTexID[1]);
       myGL.VertexPointer(2, GL_FLOAT, 0, (const GLvoid*)(16*sizeof(GLfloat)));
@@ -163,7 +172,6 @@ void FBSurfaceTIA::update()
       myGL.DrawArrays(GL_TRIANGLE_STRIP, 0, 4);
       myGL.Disable(GL_BLEND);
     }
-#endif
   }
   else
   {
@@ -171,11 +179,10 @@ void FBSurfaceTIA::update()
     myGL.TexCoordPointer(2, GL_FLOAT, 0, myCoord+8);
     myGL.DrawArrays(GL_TRIANGLE_STRIP, 0, 4);
 
-#if 0
-    if(1)//myFB.myScanlinesEnabled)
+    if(myScanlinesEnabled)
     {
       myGL.Enable(GL_BLEND);
-      myGL.Color4f(1.0f, 1.0f, 1.0f, 0.5f);
+      myGL.Color4f(1.0f, 1.0f, 1.0f, myScanlineIntensityF);
       myGL.BlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
       myGL.BindTexture(GL_TEXTURE_2D, myTexID[1]);
       myGL.VertexPointer(2, GL_FLOAT, 0, myCoord+16);
@@ -183,7 +190,6 @@ void FBSurfaceTIA::update()
       myGL.DrawArrays(GL_TRIANGLE_STRIP, 0, 4);
       myGL.Disable(GL_BLEND);
     }
-#endif
   }
 
   myGL.DisableClientState(GL_VERTEX_ARRAY);
@@ -193,6 +199,12 @@ void FBSurfaceTIA::update()
   myFB.myDirtyFlag = true;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FBSurfaceTIA::invalidate()
+{
+  SDL_FillRect(myTexture, NULL, 0);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FBSurfaceTIA::free()
 {
@@ -221,38 +233,31 @@ void FBSurfaceTIA::reload()
 
   // Base texture (@ index 0)
   myGL.BindTexture(GL_TEXTURE_2D, myTexID[0]);
-  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, myTexFilter[0]);
+  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, myTexFilter[0]);
   myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
   myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 
   // Create the texture in the most optimal format
-  myGL.TexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
-                 myTexWidth, myTexHeight, 0,
-#ifdef HAVE_GL_BGRA
-                 GL_BGRA, GL_UNSIGNED_SHORT_1_5_5_5_REV,
-#else
-                 GL_RGBA, GL_UNSIGNED_SHORT_5_5_5_1,
-#endif
+  myGL.PixelStorei(GL_UNPACK_ALIGNMENT, 1);
+  myGL.PixelStorei(GL_UNPACK_ROW_LENGTH, myPitch);
+  myGL.TexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, myTexWidth, myTexHeight, 0,
+                 GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV,
                  myTexture->pixels);
 
   // Scanline texture (@ index 1)
   myGL.BindTexture(GL_TEXTURE_2D, myTexID[1]);
-  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, myTexFilter[1]);
+  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, myTexFilter[1]);
   myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
   myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
-#ifdef HAVE_GL_BGRA
-  static uInt16 const scanline[4] = { 0x0000, 0x0000, 0x8000, 0x0000  };
+
+  static uInt32 const scanline[2] = { 0x00000000, 0xff000000 };
+  myGL.PixelStorei(GL_UNPACK_ALIGNMENT, 1);
+  myGL.PixelStorei(GL_UNPACK_ROW_LENGTH, 1);
   myGL.TexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 1, 2, 0,
-                 GL_BGRA, GL_UNSIGNED_SHORT_1_5_5_5_REV,
+                 GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV,
                  scanline);
-#else
-  static uInt16 const scanline[4] = { 0x0000, 0x0000, 0x0001, 0x0000 };
-  myGL.TexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 1, 2, 0,
-                 GL_RGBA, GL_UNSIGNED_SHORT_5_5_5_1,
-                 scanline);
-#endif
 
   // Cache vertex and texture coordinates using vertex buffer object
   if(myFB.myVBOAvailable)
@@ -264,23 +269,55 @@ void FBSurfaceTIA::reload()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FBSurfaceTIA::setFilter(const string& name)
+void FBSurfaceTIA::setScanIntensity(uInt32 intensity)
 {
-  // We only do GL_NEAREST or GL_LINEAR for now
-  GLint filter = GL_NEAREST;
-  if(name == "linear")
-    filter = GL_LINEAR;
+  myScanlineIntensityI = (GLuint)intensity;
+  myScanlineIntensityF = (GLfloat)intensity / 100;
+}
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FBSurfaceTIA::setTexInterpolation(bool enable)
+{
+  myTexFilter[0] = enable ? GL_LINEAR : GL_NEAREST;
   myGL.BindTexture(GL_TEXTURE_2D, myTexID[0]);
-  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter);
-  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter);
+  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, myTexFilter[0]);
+  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, myTexFilter[0]);
   myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
   myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FBSurfaceTIA::setScanInterpolation(bool enable)
+{
+  myTexFilter[1] = enable ? GL_LINEAR : GL_NEAREST;
+  myGL.BindTexture(GL_TEXTURE_2D, myTexID[1]);
+  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, myTexFilter[1]);
+  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, myTexFilter[1]);
+  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
+  myGL.TexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FBSurfaceTIA::updateCoords(uInt32 baseH,
+     uInt32 imgX, uInt32 imgY, uInt32 imgW, uInt32 imgH)
+{
+  myBaseH = baseH;
+  myImageX = imgX;  myImageY = imgY;
+  myImageW = imgW;  myImageH = imgH;
+
+  updateCoords();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FBSurfaceTIA::updateCoords()
 {
+  // Normal TIA rendering and TV effects use different widths
+  // We use the same buffer, and only pick the width we need
+  myBaseW = myFB.ntscEnabled() ? ATARI_NTSC_OUT_WIDTH(160) : 160;
+
+  myTexCoordW = (GLfloat) myBaseW / myTexWidth;
+  myTexCoordH = (GLfloat) myBaseH / myTexHeight;
+
   // Vertex coordinates for texture 0 (main texture)
   // Upper left (x,y)
   myCoord[0] = (GLfloat)myImageX;
@@ -330,12 +367,16 @@ void FBSurfaceTIA::updateCoords()
   // Upper right (x+w,y)
   myCoord[26] = 1.0f;
   myCoord[27] = 0.0f;
+  // Scanline repeating is sensitive to non-integral vertical resolution,
+  // so rounding is performed to eliminate it
+  // This won't be 100% accurate, but non-integral scaling isn't 100%
+  // accurate anyway
   // Lower left (x,y+h)
   myCoord[28] = 0.0f;
-  myCoord[29] = (GLfloat)(myImageH/myFB.myZoomLevel);
+  myCoord[29] = GLfloat(myImageH) / floor(((float)myImageH / myBaseH) + 0.5);
   // Lower right (x+w,y+h)
   myCoord[30] = 1.0f;
-  myCoord[31] = (GLfloat)(myImageH/myFB.myZoomLevel);
+  myCoord[31] = myCoord[29];
 
   // Cache vertex and texture coordinates using vertex buffer object
   if(myFB.myVBOAvailable)
@@ -345,4 +386,10 @@ void FBSurfaceTIA::updateCoords()
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FBSurfaceTIA::setTIAPalette(const uInt32* palette)
+{
+  myFB.myNTSCFilter.setTIAPalette(myFB, palette);
+}
+
 #endif
