@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FrameBufferGL.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: FrameBufferGL.cxx 2524 2012-06-04 15:52:56Z stephena $
 //============================================================================
 
 #ifdef DISPLAY_OPENGL
@@ -40,20 +40,18 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 FrameBufferGL::FrameBufferGL(OSystem* osystem)
   : FrameBuffer(osystem),
+    myFilterType(kNormal),
     myTiaSurface(NULL),
-    myFilterParamName("GL_NEAREST"),
     myDirtyFlag(true)
 {
   // We need a pixel format for palette value calculations
   // It's done this way (vs directly accessing a FBSurfaceGL object)
   // since the structure may be needed before any FBSurface's have
   // been created
-  SDL_Surface* s = SDL_CreateRGBSurface(SDL_SWSURFACE, 1, 1, 16,
-#ifdef HAVE_GL_BGRA
-                     0x00007c00, 0x000003e0, 0x0000001f, 0x00000000);
-#else
-                     0x0000f800, 0x000007c0, 0x0000003e, 0x00000000);
-#endif
+  // Note: alpha disabled for now, since it's not used
+  SDL_Surface* s = SDL_CreateRGBSurface(SDL_SWSURFACE, 1, 1, 32,
+                       0x00ff0000, 0x0000ff00, 0x000000ff, 0x00000000);
+
   myPixelFormat = *(s->format);
   SDL_FreeSurface(s);
 }
@@ -105,6 +103,7 @@ bool FrameBufferGL::loadFuncs(GLFunctionality functionality)
         OGL_INIT(Ortho,void,glOrtho,(GLdouble, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble));
         OGL_INIT(Viewport,void,glViewport,(GLint, GLint, GLsizei, GLsizei));
         OGL_INIT(LoadIdentity,void,glLoadIdentity,(void));
+        OGL_INIT(Translatef,void,glTranslatef,(GLfloat,GLfloat,GLfloat));
         OGL_INIT(EnableClientState,void,glEnableClientState,(GLenum));
         OGL_INIT(DisableClientState,void,glDisableClientState,(GLenum));
         OGL_INIT(VertexPointer,void,glVertexPointer,(GLint,GLenum,GLsizei,const GLvoid*));
@@ -131,10 +130,6 @@ bool FrameBufferGL::loadFuncs(GLFunctionality functionality)
         OGL_INIT(BufferData,void,glBufferData,(GLenum,GLsizei,const void*,GLenum));
         OGL_INIT(DeleteBuffers,void,glDeleteBuffers,(GLsizei, const GLuint*));
         break;  // kGL_VBO
-
-      case kGL_FBO:
-        return false;  // TODO - implement this
-        break;  // kGL_FBO
     }
   }
   else
@@ -170,10 +165,10 @@ bool FrameBufferGL::initSubsystem(VideoMode& mode)
     return false;
 
   // Now check to see what color components were actually created
-  SDL_GL_GetAttribute( SDL_GL_RED_SIZE, (int*)&myRGB[0] );
-  SDL_GL_GetAttribute( SDL_GL_GREEN_SIZE, (int*)&myRGB[1] );
-  SDL_GL_GetAttribute( SDL_GL_BLUE_SIZE, (int*)&myRGB[2] );
-  SDL_GL_GetAttribute( SDL_GL_ALPHA_SIZE, (int*)&myRGB[3] );
+  SDL_GL_GetAttribute( SDL_GL_RED_SIZE, &myRGB[0] );
+  SDL_GL_GetAttribute( SDL_GL_GREEN_SIZE, &myRGB[1] );
+  SDL_GL_GetAttribute( SDL_GL_BLUE_SIZE, &myRGB[2] );
+  SDL_GL_GetAttribute( SDL_GL_ALPHA_SIZE, &myRGB[3] );
 
   return true;
 }
@@ -188,35 +183,25 @@ string FrameBufferGL::about() const
       << "  Version:    " << p_gl.GetString(GL_VERSION) << endl
       << "  Color:      " << myDepth << " bit, " << myRGB[0] << "-"
       << myRGB[1] << "-"  << myRGB[2] << "-" << myRGB[3] << ", "
-#ifdef HAVE_GL_BGRA
       << "GL_BGRA" << endl
-#else
-      << "GL_RGBA" << endl
-#endif
-      << "  Filter:     " << myFilterParamName << endl
-      << "  Extensions: ";
-  if(myVBOAvailable) out << "VBO ";
-  if(myFBOAvailable) out << "FBO ";
-  out << endl;
+      << "  Extensions: VBO " << (myVBOAvailable ? "enabled" : "disabled")
+      << endl;
   return out.str();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool FrameBufferGL::setVidMode(VideoMode& mode)
 {
-  bool inUIMode =
-    myOSystem->eventHandler().state() == EventHandler::S_LAUNCHER ||
-    myOSystem->eventHandler().state() == EventHandler::S_DEBUGGER;
+  bool inTIAMode =
+    myOSystem->eventHandler().state() != EventHandler::S_LAUNCHER &&
+    myOSystem->eventHandler().state() != EventHandler::S_DEBUGGER;
 
-  // Grab the initial width and height before it's updated below
-  uInt32 baseWidth = mode.image_w / mode.gfxmode.zoom;
+  // Grab the initial height before it's updated below
+  // We need it for the creating the TIA surface
   uInt32 baseHeight = mode.image_h / mode.gfxmode.zoom;
 
-  // Set the zoom level
-  myZoomLevel = mode.gfxmode.zoom;
-
   // Aspect ratio and fullscreen stretching only applies to the TIA
-  if(!inUIMode)
+  if(inTIAMode)
   {
     // Aspect ratio (depends on whether NTSC or PAL is detected)
     // Not available in 'small' resolutions
@@ -229,18 +214,36 @@ bool FrameBufferGL::setVidMode(VideoMode& mode)
     }
 
     // Fullscreen mode stretching
-    if(fullScreen() && myOSystem->settings().getBool("gl_fsmax") &&
+    if(fullScreen() &&
        (mode.image_w < mode.screen_w) && (mode.image_h < mode.screen_h))
     {
       float stretchFactor = 1.0;
       float scaleX = float(mode.image_w) / mode.screen_w;
       float scaleY = float(mode.image_h) / mode.screen_h;
 
-      if(scaleX > scaleY)
-        stretchFactor = float(mode.screen_w) / mode.image_w;
+      // Scale to actual or integral factors
+      if(myOSystem->settings().getBool("gl_fsscale"))
+      {
+        // Scale to full (non-integral) available space
+        if(scaleX > scaleY)
+          stretchFactor = float(mode.screen_w) / mode.image_w;
+        else
+          stretchFactor = float(mode.screen_h) / mode.image_h;
+      }
       else
-        stretchFactor = float(mode.screen_h) / mode.image_h;
-
+      {
+        // Only scale to an integral amount
+        if(scaleX > scaleY)
+        {
+          int bw = mode.image_w / mode.gfxmode.zoom;
+          stretchFactor = float(int(mode.screen_w / bw) * bw) / mode.image_w;
+        }
+        else
+        {
+          int bh = mode.image_h / mode.gfxmode.zoom;
+          stretchFactor = float(int(mode.screen_h / bh) * bh) / mode.image_h;
+        }
+      }
       mode.image_w = (Uint16) (stretchFactor * mode.image_w);
       mode.image_h = (Uint16) (stretchFactor * mode.image_h);
     }
@@ -274,14 +277,7 @@ bool FrameBufferGL::setVidMode(VideoMode& mode)
 
   // Load OpenGL function pointers
   if(loadFuncs(kGL_BASIC))
-  {
-    // Grab OpenGL version number
-    string version((const char *)p_gl.GetString(GL_VERSION));
-    myGLVersion = atof(version.substr(0, 3).c_str());
-
     myVBOAvailable = myOSystem->settings().getBool("gl_vbo") && loadFuncs(kGL_VBO);
-    myFBOAvailable = false;
-  }
   else
     return false;
 
@@ -300,63 +296,35 @@ bool FrameBufferGL::setVidMode(VideoMode& mode)
   p_gl.Ortho(0.0, mode.screen_w, mode.screen_h, 0.0, -1.0, 1.0);
   p_gl.MatrixMode(GL_MODELVIEW);
   p_gl.LoadIdentity();
+  p_gl.Translatef(0.375, 0.375, 0.0);  // fix scanline mis-draw issues
 
-/*
-cerr << "dimensions: " << (fullScreen() ? "(full)" : "") << endl
-	<< "  screen w = " << mode.screen_w << endl
-	<< "  screen h = " << mode.screen_h << endl
-	<< "  image x  = " << mode.image_x << endl
-	<< "  image y  = " << mode.image_y << endl
-	<< "  image w  = " << mode.image_w << endl
-	<< "  image h  = " << mode.image_h << endl
-	<< "  base w   = " << baseWidth << endl
-	<< "  base h   = " << baseHeight << endl
-	<< endl;
-*/
+//cerr << "dimensions: " << (fullScreen() ? "(full)" : "") << endl << mode << endl;
 
-  ////////////////////////////////////////////////////////////////////
-  // Note that the following must be done in the order given
-  // Basically, all surfaces must first be free'd before being
-  // recreated
-  // So, we delete the TIA surface first, then reset all other surfaces
-  // (which frees all surfaces and then reloads all surfaces), then
-  // re-create the TIA surface (if necessary)
-  // In this way, all free()'s come before all reload()'s
-  ////////////////////////////////////////////////////////////////////
-
-  // We try to re-use the TIA surface whenever possible
-  if(!inUIMode && !(myTiaSurface &&
-      myTiaSurface->getWidth() == mode.image_w &&
-      myTiaSurface->getHeight() == mode.image_h))
+  // The framebuffer only takes responsibility for TIA surfaces
+  // Other surfaces (such as the ones used for dialogs) are allocated
+  // in the Dialog class
+  if(inTIAMode)
   {
-    delete myTiaSurface;  myTiaSurface = NULL;
+    // Since we have free hardware stretching, the base TIA surface is created
+    // only once, and its texture coordinates changed when we want to draw a
+    // smaller or larger image
+    if(!myTiaSurface)
+      myTiaSurface = new FBSurfaceTIA(*this);
+
+    myTiaSurface->updateCoords(baseHeight, mode.image_x, mode.image_y,
+                               mode.image_w, mode.image_h);
+
+    myTiaSurface->enableScanlines(ntscEnabled());
+    myTiaSurface->setScanIntensity(myOSystem->settings().getInt("tv_scanlines"));
+    myTiaSurface->setTexInterpolation(myOSystem->settings().getBool("gl_inter"));
+    myTiaSurface->setScanInterpolation(myOSystem->settings().getBool("tv_scaninter"));
+    myTiaSurface->setTIA(myOSystem->console().tia());
   }
 
   // Any previously allocated textures currently in use by various UI items
   // need to be refreshed as well (only seems to be required for OSX)
   resetSurfaces(myTiaSurface);
 
-  // The framebuffer only takes responsibility for TIA surfaces
-  // Other surfaces (such as the ones used for dialogs) are allocated
-  // in the Dialog class
-  if(!inUIMode)
-  {
-    // The actual TIA image is only half of that specified by baseWidth
-    // The stretching can be done in hardware now that the TIA surface
-    // and other UI surfaces are no longer tied together
-    // Note that this may change in the future, when we add more
-    // complex filters/scalers, but for now it's fine
-    //
-    // Also note that TV filtering is always available since we'll always
-    // have access to Blargg filtering
-    if(!myTiaSurface)
-      myTiaSurface = new FBSurfaceTIA(*this, baseWidth>>1, baseHeight,
-            mode.image_x, mode.image_y, mode.image_w, mode.image_h);
-
-    myTiaSurface->setFilter(myOSystem->settings().getString("gl_filter"));
-    myTiaSurface->setTIA(myOSystem->console().tia());
-  }
-
   return true;
 }
 
@@ -364,6 +332,8 @@ cerr << "dimensions: " << (fullScreen() ? "(full)" : "") << endl
 void FrameBufferGL::invalidate()
 {
   p_gl.Clear(GL_COLOR_BUFFER_BIT);
+  if(myTiaSurface)
+    myTiaSurface->invalidate();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -387,10 +357,64 @@ void FrameBufferGL::postFrameUpdate()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FrameBufferGL::enablePhosphor(bool enable, int blend)
 {
-  myUsePhosphor   = enable;
-  myPhosphorBlend = blend;
+  if(myTiaSurface)
+  {
+    myUsePhosphor   = enable;
+    myPhosphorBlend = blend;
+    myFilterType = FilterType(enable ? myFilterType | 0x01 : myFilterType & 0x10);
+    myRedrawEntireFrame = true;
+  }
+}
 
-  myRedrawEntireFrame = true;
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FrameBufferGL::enableNTSC(bool enable)
+{
+  if(myTiaSurface)
+  {
+    myFilterType = FilterType(enable ? myFilterType | 0x10 : myFilterType & 0x01);
+    myTiaSurface->updateCoords();
+
+    myTiaSurface->enableScanlines(ntscEnabled());
+    myTiaSurface->setScanIntensity(myOSystem->settings().getInt("tv_scanlines"));
+    myTiaSurface->setTexInterpolation(myOSystem->settings().getBool("gl_inter"));
+    myTiaSurface->setScanInterpolation(myOSystem->settings().getBool("tv_scaninter"));
+
+    myRedrawEntireFrame = true;
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt32 FrameBufferGL::enableScanlines(int relative, int absolute)
+{
+  int intensity = myTiaSurface->myScanlineIntensityI;
+  if(myTiaSurface)
+  {
+    if(relative == 0)  intensity = absolute;
+    else               intensity += relative;
+    intensity = BSPF_max(0, intensity);
+    intensity = BSPF_min(100, intensity);
+
+    myTiaSurface->setScanIntensity(intensity);
+    myRedrawEntireFrame = true;
+  }
+  return intensity;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FrameBufferGL::enableScanlineInterpolation(bool enable)
+{
+  if(myTiaSurface)
+  {
+    myTiaSurface->setScanInterpolation(enable);
+    myRedrawEntireFrame = true;
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FrameBufferGL::setTIAPalette(const uInt32* palette)
+{
+  FrameBuffer::setTIAPalette(palette);
+  myTiaSurface->setTIAPalette(palette);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -417,13 +441,7 @@ void FrameBufferGL::scanline(uInt32 row, uInt8* data) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool FrameBufferGL::myLibraryLoaded = false;
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-float FrameBufferGL::myGLVersion = 0.0;
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool FrameBufferGL::myVBOAvailable = false;
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool FrameBufferGL::myFBOAvailable = false;
-
 #endif  // DISPLAY_OPENGL
