@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FrameBufferSoft.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: FrameBufferSoft.cxx 2488 2012-05-20 18:26:59Z stephena $
 //============================================================================
 
 #include <sstream>
@@ -34,6 +34,7 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 FrameBufferSoft::FrameBufferSoft(OSystem* osystem)
   : FrameBuffer(osystem),
+    myZoomLevel(2),
     myRenderType(kSoftZoom_16),
     myTiaDirty(false),
     myInUIMode(false),
@@ -131,7 +132,6 @@ bool FrameBufferSoft::setVidMode(VideoMode& mode)
   mode.screen_w = myScreen->w;
   mode.screen_h = myScreen->h;
   myZoomLevel = mode.gfxmode.zoom;
-// FIXME - look at gfxmode directly
 
   // Erase old rects, since they've probably been scaled for
   // a different sized screen
@@ -587,10 +587,10 @@ void FBSurfaceSoft::fillRect(uInt32 x, uInt32 y, uInt32 w, uInt32 h, uInt32 colo
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FBSurfaceSoft::drawChar(const GUI::Font* font, uInt8 chr,
+void FBSurfaceSoft::drawChar(const GUI::Font& font, uInt8 chr,
                              uInt32 tx, uInt32 ty, uInt32 color)
 {
-  const FontDesc& desc = font->desc();
+  const FontDesc& desc = font.desc();
 
   // If this character is not included in the font, use the default char.
   if(chr < desc.firstchar || chr >= desc.firstchar + desc.size)
@@ -750,7 +750,6 @@ void FBSurfaceSoft::addDirtyRect(uInt32 x, uInt32 y, uInt32 w, uInt32 h)
   if(myIsBaseSurface)
   {
     // Add a dirty rect to the UI rectangle list
-    // TODO - intelligent merging of rectangles, to avoid overlap
     SDL_Rect temp;
     temp.x = x + myXOrig;  temp.y = y + myYOrig;  temp.w = w;  temp.h = h;
     myFB.myRectList->add(&temp);
