@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SoundSDL.cxx 2342 2012-01-07 20:51:03Z stephena $
+// $Id: SoundSDL.cxx 2499 2012-05-25 12:41:19Z stephena $
 //============================================================================
 
 #ifdef SOUND_SUPPORT
@@ -397,20 +397,18 @@ bool SoundSDL::save(Serializer& out) const
       reg6 = myTIASound.get(0x1a);
     }
 
-    out.putByte((char)reg1);
-    out.putByte((char)reg2);
-    out.putByte((char)reg3);
-    out.putByte((char)reg4);
-    out.putByte((char)reg5);
-    out.putByte((char)reg6);
+    out.putByte(reg1);
+    out.putByte(reg2);
+    out.putByte(reg3);
+    out.putByte(reg4);
+    out.putByte(reg5);
+    out.putByte(reg6);
 
     out.putInt(myLastRegisterSetCycle);
   }
-  catch(const char* msg)
+  catch(...)
   {
-    ostringstream buf;
-    buf << "ERROR: SoundSDL::save" << endl << "  " << msg << endl;
-    myOSystem->logMessage(buf.str(), 0);
+    myOSystem->logMessage("ERROR: SoundSDL::save", 0);
     return false;
   }
 
@@ -425,12 +423,12 @@ bool SoundSDL::load(Serializer& in)
     if(in.getString() != name())
       return false;
 
-    uInt8 reg1 = (uInt8) in.getByte(),
-          reg2 = (uInt8) in.getByte(),
-          reg3 = (uInt8) in.getByte(),
-          reg4 = (uInt8) in.getByte(),
-          reg5 = (uInt8) in.getByte(),
-          reg6 = (uInt8) in.getByte();
+    uInt8 reg1 = in.getByte(),
+          reg2 = in.getByte(),
+          reg3 = in.getByte(),
+          reg4 = in.getByte(),
+          reg5 = in.getByte(),
+          reg6 = in.getByte();
 
     myLastRegisterSetCycle = (Int32) in.getInt();
 
@@ -449,11 +447,9 @@ bool SoundSDL::load(Serializer& in)
       if(!myIsMuted) SDL_PauseAudio(0);
     }
   }
-  catch(const char* msg)
+  catch(...)
   {
-    ostringstream buf;
-    buf << "ERROR: SoundSDL::load" << endl << "  " << msg << endl;
-    myOSystem->logMessage(buf.str(), 0);
+    myOSystem->logMessage("ERROR: SoundSDL::load", 0);
     return false;
   }
 
