@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: mainSDL.cxx 2333 2012-01-04 14:54:21Z stephena $
+// $Id: mainSDL.cxx 2477 2012-05-16 20:52:33Z stephena $
 //============================================================================
 
 #include <SDL.h>
@@ -66,6 +66,7 @@ OSystem* theOSystem = (OSystem*) NULL;
 int Cleanup()
 {
   theOSystem->logMessage("Cleanup from mainSDL\n", 2);
+  theOSystem->saveConfig();
 
   if(theOSystem)
     delete theOSystem;
@@ -103,7 +104,7 @@ int main(int argc, char* argv[])
   #error Unsupported platform!
 #endif
 
-  theOSystem->settings().loadConfig();
+  theOSystem->loadConfig();
   theOSystem->logMessage("Loading config options ...\n", 2);
 
   // Take care of commandline arguments
@@ -137,7 +138,7 @@ int main(int argc, char* argv[])
   {
     theOSystem->logMessage("Showing output from 'rominfo' ...\n", 2);
     FilesystemNode romnode(romfile);
-    if(argc > 1 && romnode.exists() && !romnode.isDirectory())
+    if(argc > 1 && romnode.exists() && romnode.isFile())
       theOSystem->logMessage(theOSystem->getROMInfo(romfile), 0);
     else
       theOSystem->logMessage("ERROR: ROM doesn't exist\n", 0);
@@ -171,14 +172,9 @@ int main(int argc, char* argv[])
   if(romfile == "" || romnode.isDirectory())
   {
     theOSystem->logMessage("Attempting to use ROM launcher ...\n", 2);
-    if(theOSystem->settings().getBool("uselauncher"))
-    {
-      bool launcherOpened = romfile != "" ?
-        theOSystem->createLauncher(romnode.getPath()) : theOSystem->createLauncher();
-      if(!launcherOpened)
-        return Cleanup();
-    }
-    else
+    bool launcherOpened = romfile != "" ?
+      theOSystem->createLauncher(romnode.getPath()) : theOSystem->createLauncher();
+    if(!launcherOpened)
     {
       theOSystem->logMessage("Launcher could not be started, showing usage\n", 2);
       theOSystem->settings().usage();
