@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DebuggerParser.cxx 2254 2011-06-11 22:45:50Z stephena $
+// $Id: DebuggerParser.cxx 2318 2011-12-31 21:56:36Z stephena $
 //============================================================================
 
 #include <fstream>
@@ -135,7 +135,7 @@ string DebuggerParser::exec(const FilesystemNode& file)
   {
     ifstream in(file.getPath().c_str());
     if(!in.is_open())
-      return red("autoexec file \'" + file.getPath(false) + "\' not found");
+      return red("autoexec file \'" + file.getRelativePath() + "\' not found");
 
     ostringstream buf;
     int count = 0;
@@ -149,12 +149,12 @@ string DebuggerParser::exec(const FilesystemNode& file)
       count++;
     }
     buf << "Executed " << debugger->valueToString(count) << " commands from \""
-        << file.getPath(false) << "\"";
+        << file.getRelativePath() << "\"";
 
     return buf.str();
   }
   else
-    return red("autoexec file \'" + file.getPath(false) + "\' not found");
+    return red("autoexec file \'" + file.getRelativePath() + "\' not found");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -265,7 +265,11 @@ int DebuggerParser::decipher_arg(const string &str)
       } else { // must be hex.
         result = 0;
         while(*a != '\0') {
-          int hex = Debugger::conv_hex_digit(*a++);
+          int hex = -1;
+          char d = *a++;
+          if(d >= '0' && d <= '9')  hex = d - '0';
+          else if(d >= 'a' && d <= 'f') hex = d - 'a' + 10;
+          else if(d >= 'A' && d <= 'F') hex = d - 'A' + 10;
           if(hex < 0)
             return -1;
 
@@ -288,31 +292,25 @@ int DebuggerParser::decipher_arg(const string &str)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string DebuggerParser::showWatches()
 {
-  string ret;
-  char buf[10];
-
-  for(unsigned int i=0; i<watches.size(); i++) {
-    if(watches[i] != "") {
+  ostringstream buf;
+  for(unsigned int i = 0; i < watches.size(); i++)
+  {
+    if(watches[i] != "")
+    {
       // Clear the args, since we're going to pass them to eval()
       argStrings.clear();
       args.clear();
 
-      sprintf(buf, "%d", i+1);
       argCount = 1;
       argStrings.push_back(watches[i]);
       args.push_back(decipher_arg(argStrings[0]));
-      if(args[0] < 0) {
-        ret += "BAD WATCH ";
-        ret += buf;
-        ret += ": " + argStrings[0] + "\n";
-      } else {
-        ret += " watch #";
-        ret += buf;
-        ret += " (" + argStrings[0] + ") -> " + eval() + "\n";
-      }
+      if(args[0] < 0)
+        buf << "BAD WATCH " << (i+1) << ": " << argStrings[0] << endl;
+      else
+        buf << " watch #" << (i+1) << " (" << argStrings[0] << ") -> " << eval() << endl;
     }
   }
-  return ret;
+  return buf.str();
 }
 
 
@@ -525,30 +523,38 @@ cerr << "curCount         = " << curCount << endl
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string DebuggerParser::eval()
 {
-  char buf[50];
-  string ret;
-  for(int i=0; i<argCount; i++) {
-    // TODO - technically, we should determine if the label is read or write
-    const string& label = debugger->cartDebug().getLabel(args[i], true);
-    if(label != "") {
-      ret += label;
-      ret += ": ";
+  ostringstream buf;
+  for(int i = 0; i < argCount; ++i)
+  {
+    string rlabel = debugger->cartDebug().getLabel(args[i], true);
+    string wlabel = debugger->cartDebug().getLabel(args[i], false);
+    bool validR = rlabel != "" && rlabel[0] != '$',
+         validW = wlabel != "" && wlabel[0] != '$';
+    if(validR && validW)
+    {
+      if(rlabel == wlabel)
+        buf << rlabel << "(R/W): ";
+      else
+        buf << rlabel << "(R) / " << wlabel << "(W): ";
     }
-    ret += "$";
-    if(args[i] < 0x100) {
-      ret += Debugger::to_hex_8(args[i]);
-      ret += " %";
-      ret += Debugger::to_bin_8(args[i]);
-    } else {
-      ret += Debugger::to_hex_16(args[i]);
-      ret += " %";
-      ret += Debugger::to_bin_16(args[i]);
-    }
-    sprintf(buf, " #%d", args[i]);
-    ret += buf;
-    if(i != argCount - 1) ret += "\n";
+    else if(validR)
+      buf << rlabel << "(R): ";
+    else if(validW)
+      buf << wlabel << "(W): ";
+
+    if(args[i] < 0x100)
+      buf << "$" << debugger->valueToString(args[i], kBASE_16_2)
+          << " %" << debugger->valueToString(args[i], kBASE_2_8);
+    else
+      buf << "$" << debugger->valueToString(args[i], kBASE_16_4)
+          << " %" << debugger->valueToString(args[i], kBASE_2_16);
+
+    buf << " #" << (int) args[i]; 
+    if(i != argCount - 1)
+      buf << endl;
   }
-  return ret;
+
+  return buf.str();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
