@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DiStella.cxx 2417 2012-03-16 13:19:29Z stephena $
+// $Id: DiStella.cxx 2502 2012-05-27 13:07:38Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -458,7 +458,7 @@ void DiStella::disasm(uInt32 distart, int pass)
           }
           else if (pass == 3)
           {
-            if (ad < 0x100)
+            if (ad < 0x100 && settings.fflag)
               nextline << ".w  ";
             else
               nextline << "    ";
@@ -473,7 +473,7 @@ void DiStella::disasm(uInt32 distart, int pass)
               nextline << CartDebug::ourIOMnemonic[ad-0x280];
               nextlinebytes << HEX2 << (int)(ad&0xff) << " " << HEX2 << (int)(ad>>8);
             }
-            else if (labfound == 4)
+            else if (labfound == 4 && settings.rflag)
             {
               int tmp = (ad & myAppData.end)+myOffset;
               USER_OR_AUTO_LABEL("", tmp, "");
@@ -530,7 +530,7 @@ void DiStella::disasm(uInt32 distart, int pass)
           }
           else if (pass == 3)
           {
-            if (ad < 0x100)
+            if (ad < 0x100 && settings.fflag)
               nextline << ".wx ";
             else
               nextline << "    ";
@@ -545,7 +545,7 @@ void DiStella::disasm(uInt32 distart, int pass)
               nextline << CartDebug::ourIOMnemonic[ad-0x280] << ",X";
               nextlinebytes << HEX2 << (int)(ad&0xff) << " " << HEX2 << (int)(ad>>8);
             }
-            else if (labfound == 4)
+            else if (labfound == 4 && settings.rflag)
             {
               int tmp = (ad & myAppData.end)+myOffset;
               USER_OR_AUTO_LABEL("", tmp, ",X");
@@ -574,7 +574,7 @@ void DiStella::disasm(uInt32 distart, int pass)
           }
           else if (pass == 3)
           {
-            if (ad < 0x100)
+            if (ad < 0x100 && settings.fflag)
               nextline << ".wy ";
             else
               nextline << "    ";
@@ -589,7 +589,7 @@ void DiStella::disasm(uInt32 distart, int pass)
               nextline << CartDebug::ourIOMnemonic[ad-0x280] << ",Y";
               nextlinebytes << HEX2 << (int)(ad&0xff) << " " << HEX2 << (int)(ad>>8);
             }
-            else if (labfound == 4)
+            else if (labfound == 4 && settings.rflag)
             {
               int tmp = (ad & myAppData.end)+myOffset;
               USER_OR_AUTO_LABEL("", tmp, ",Y");
@@ -705,7 +705,7 @@ void DiStella::disasm(uInt32 distart, int pass)
           }
           else if (pass == 3)
           {
-            if (ad < 0x100)
+            if (ad < 0x100 && settings.fflag)
               nextline << ".ind ";
             else
               nextline << "     ";
@@ -812,13 +812,9 @@ int DiStella::mark(uInt32 address, uInt8 mask, bool directive)
     ===========================================================
   -----------------------------------------------------------------------*/
 
-  if (address >= myOffset && address <= myAppData.end + myOffset)
-  {
-    myLabels[address-myOffset] = myLabels[address-myOffset] | mask;
-    if(directive)  myDirectives[address-myOffset] = mask;
-    return 1;
-  }
-  else if (address <= 0x3f)
+  // Check for equates before ROM/ZP-RAM accesses, because the original logic
+  // of Distella assumed either equates or ROM; it didn't take ZP-RAM into account
+  if (address <= 0x3f)
   {
     return 2;
   }
@@ -826,7 +822,13 @@ int DiStella::mark(uInt32 address, uInt8 mask, bool directive)
   {
     return 3;
   }
-  else if (address > 0x1000)
+  else if (address >= myOffset && address <= myAppData.end + myOffset)
+  {
+    myLabels[address-myOffset] = myLabels[address-myOffset] | mask;
+    if(directive)  myDirectives[address-myOffset] = mask;
+    return 1;
+  }
+  else if (address > 0x1000 && myOffset != 0)  // Exclude zero-page accesses
   {
     /* 2K & 4K case */
     myLabels[address & myAppData.end] = myLabels[address & myAppData.end] | mask;
