@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DataGridOpsWidget.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: DataGridOpsWidget.cxx 2438 2012-04-11 16:10:24Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -37,8 +37,8 @@ DataGridOpsWidget::DataGridOpsWidget(GuiObject* boss, const GUI::Font& font,
 {
   _type = kDataGridOpsWidget;
 
-  const int bwidth  = _font->getMaxCharWidth() * 4,
-            bheight = _font->getFontHeight() + 3,
+  const int bwidth  = _font.getMaxCharWidth() * 4,
+            bheight = _font.getFontHeight() + 3,
             space = 6;
   int xpos, ypos;
 
