@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DataGridWidget.cxx 2260 2011-07-03 17:08:42Z stephena $
+// $Id: DataGridWidget.cxx 2417 2012-03-16 13:19:29Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -47,6 +47,7 @@ DataGridWidget::DataGridWidget(GuiObject* boss, const GUI::Font& font,
     _bits(bits),
     _base(base),
     _selectedItem(0),
+    _currentKeyDown(KBDK_UNKNOWN),
     _opsWidget(NULL),
     _scrollBar(NULL)
 {
@@ -55,8 +56,6 @@ DataGridWidget::DataGridWidget(GuiObject* boss, const GUI::Font& font,
   _type = kDataGridWidget;
   _editMode = false;
 
-  _currentKeyDown = 0;
-
   // The item is selected, thus _bgcolor is used to draw the caret and
   // _textcolorhi to erase it
   _caretInverse = true;
@@ -146,6 +145,20 @@ void DataGridWidget::setList(int a, int v, bool c)
   setList(alist, vlist, changed);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DataGridWidget::setList(int a, int v)
+{
+  IntArray alist, vlist;
+  BoolArray changed;
+
+  alist.push_back(a);
+  vlist.push_back(v);
+  bool diff = _addrList.size() == 1 ? getSelectedValue() != v : false;
+  changed.push_back(diff);
+
+  setList(alist, vlist, changed);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DataGridWidget::setHiliteList(const BoolArray& hilitelist)
 {
@@ -268,11 +281,11 @@ int DataGridWidget::findItem(int x, int y)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool DataGridWidget::handleKeyDown(int ascii, int keycode, int modifiers)
+bool DataGridWidget::handleKeyDown(StellaKey key, StellaMod mod, char ascii)
 {
   // Ignore all mod keys
-  if(instance().eventHandler().kbdControl(modifiers) ||
-     instance().eventHandler().kbdAlt(modifiers))
+  if(instance().eventHandler().kbdControl(mod) ||
+     instance().eventHandler().kbdAlt(mod))
     return true;
 
   bool handled = true;
@@ -281,17 +294,16 @@ bool DataGridWidget::handleKeyDown(int ascii, int keycode, int modifiers)
   if (_editMode)
   {
     // Class EditableWidget handles all text editing related key presses for us
-    handled = EditableWidget::handleKeyDown(ascii, keycode, modifiers);
+    handled = EditableWidget::handleKeyDown(key, mod, ascii);
     if(handled)
       setDirty(); draw();
   }
   else
   {
     // not editmode
-    switch(ascii)
+    switch(key)
     {
-      case '\n':  // enter/return
-      case '\r':
+      case KBDK_RETURN:
         if (_currentRow >= 0 && _currentCol >= 0)
         {
           dirty = true;
@@ -300,7 +312,7 @@ bool DataGridWidget::handleKeyDown(int ascii, int keycode, int modifiers)
         }
         break;
 
-      case 256+17:  // up arrow
+      case KBDK_UP:
         if (_currentRow > 0)
         {
           _currentRow--;
@@ -314,7 +326,7 @@ bool DataGridWidget::handleKeyDown(int ascii, int keycode, int modifiers)
         }
         break;
 
-      case 256+18:  // down arrow
+      case KBDK_DOWN:
         if (_currentRow < (int) _rows - 1)
         {
           _currentRow++;
@@ -328,7 +340,7 @@ bool DataGridWidget::handleKeyDown(int ascii, int keycode, int modifiers)
         }
         break;
 
-      case 256+20:  // left arrow
+      case KBDK_LEFT:
         if (_currentCol > 0)
         {
           _currentCol--;
@@ -342,7 +354,7 @@ bool DataGridWidget::handleKeyDown(int ascii, int keycode, int modifiers)
         }
         break;
 
-      case 256+19:  // right arrow
+      case KBDK_RIGHT:
         if (_currentCol < (int) _cols - 1)
         {
           _currentCol++;
@@ -356,8 +368,8 @@ bool DataGridWidget::handleKeyDown(int ascii, int keycode, int modifiers)
         }
         break;
 
-      case 256+24:  // pageup
-        if(instance().eventHandler().kbdShift(modifiers) && _scrollBar)
+      case KBDK_PAGEUP:
+        if(instance().eventHandler().kbdShift(mod) && _scrollBar)
           handleMouseWheel(0, 0, -1);
         else if (_currentRow > 0)
         {
@@ -366,8 +378,8 @@ bool DataGridWidget::handleKeyDown(int ascii, int keycode, int modifiers)
         }
         break;
 
-      case 256+25:	// pagedown
-        if(instance().eventHandler().kbdShift(modifiers) && _scrollBar)
+      case KBDK_PAGEDOWN:
+        if(instance().eventHandler().kbdShift(mod) && _scrollBar)
           handleMouseWheel(0, 0, +1);
         else if (_currentRow < (int) _rows - 1)
         {
@@ -376,7 +388,7 @@ bool DataGridWidget::handleKeyDown(int ascii, int keycode, int modifiers)
         }
         break;
 
-      case 256+22:  // home
+      case KBDK_HOME:
         if (_currentCol > 0)
         {
           _currentCol = 0;
@@ -384,7 +396,7 @@ bool DataGridWidget::handleKeyDown(int ascii, int keycode, int modifiers)
         }
         break;
 
-      case 256+23:  // end
+      case KBDK_END:
         if (_currentCol < (int) _cols - 1)
         {
           _currentCol = _cols - 1;
@@ -392,48 +404,58 @@ bool DataGridWidget::handleKeyDown(int ascii, int keycode, int modifiers)
         }
         break;
 
-      case 'n': // negate
+      case KBDK_n: // negate
         if(_editable)
           negateCell();
         break;
 
-      case 'i': // invert
-      case '!':
-        if(_editable)
-          invertCell();
-        break;
-
-      case '-': // decrement
-        if(_editable)
-          decrementCell();
-        break;
-
-      case '+': // increment
-      case '=':
-        if(_editable)
-          incrementCell();
-        break;
-
-      case '<': // shift left
-      case ',':
-        if(_editable)
-          lshiftCell();
-        break;
-
-      case '>': // shift right
-      case '.':
-        if(_editable)
-          rshiftCell();
-        break;
-
-      case 'z': // zero
-        if(_editable)
-          zeroCell();
-        break;
-
       default:
         handled = false;
     }
+    if(!handled)
+    {
+      handled = true;
+
+      switch(ascii)
+      {
+        case 'i': // invert
+        case '!':
+          if(_editable)
+            invertCell();
+          break;
+
+        case '-': // decrement
+          if(_editable)
+            decrementCell();
+          break;
+
+        case '+': // increment
+        case '=':
+          if(_editable)
+            incrementCell();
+          break;
+
+        case '<': // shift left
+        case ',':
+          if(_editable)
+            lshiftCell();
+          break;
+
+        case '>': // shift right
+        case '.':
+          if(_editable)
+            rshiftCell();
+          break;
+
+        case 'z': // zero
+          if(_editable)
+            zeroCell();
+          break;
+
+        default:
+          handled = false;
+      }
+    }
   }
 
   if (dirty)
@@ -447,15 +469,15 @@ bool DataGridWidget::handleKeyDown(int ascii, int keycode, int modifiers)
     setDirty(); draw();
   }
 
-  _currentKeyDown = keycode;
+  _currentKeyDown = key;
   return handled;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool DataGridWidget::handleKeyUp(int ascii, int keycode, int modifiers)
+bool DataGridWidget::handleKeyUp(StellaKey key, StellaMod mod, char ascii)
 {
-  if (keycode == _currentKeyDown)
-    _currentKeyDown = 0;
+  if (key == _currentKeyDown)
+    _currentKeyDown = KBDK_UNKNOWN;
   return true;
 }
 
@@ -615,7 +637,7 @@ void DataGridWidget::startEditMode()
   if (_editable && !_editMode && _selectedItem >= 0)
   {
     _editMode = true;
-    setEditString("");  // Erase current entry when starting editing
+    setEditString("", true );  // Erase current entry when starting editing
   }
 }
 
