@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DebuggerDialog.cxx 2234 2011-05-26 16:14:46Z stephena $
+// $Id: DebuggerDialog.cxx 2356 2012-01-14 22:00:54Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -78,12 +78,12 @@ void DebuggerDialog::loadConfig()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DebuggerDialog::handleKeyDown(int ascii, int keycode, int modifiers)
+void DebuggerDialog::handleKeyDown(StellaKey key, StellaMod mod, char ascii)
 {
-  bool handled = instance().eventHandler().kbdAlt(modifiers);
+  bool handled = instance().eventHandler().kbdAlt(mod);
   if(handled)
   {
-    switch(keycode)
+    switch(ascii)
     {
       case 's':
         doStep();
@@ -106,7 +106,7 @@ void DebuggerDialog::handleKeyDown(int ascii, int keycode, int modifiers)
     }
   }
   if(!handled)
-    Dialog::handleKeyDown(ascii, keycode, modifiers);
+    Dialog::handleKeyDown(key, mod, ascii);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
