@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: PromptWidget.cxx 2356 2012-01-14 22:00:54Z stephena $
+// $Id: PromptWidget.cxx 2438 2012-04-11 16:10:24Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -104,7 +104,7 @@ void PromptWidget::drawWidget(bool hilite)
       } else {
         fgcolor = c >> 8;
       }
-      s.drawChar(&instance().consoleFont(), c & 0x7f, x, y, fgcolor);
+      s.drawChar(instance().consoleFont(), c & 0x7f, x, y, fgcolor);
       x += _kConsoleCharWidth;
     }
     y += _kConsoleLineHeight;
@@ -833,7 +833,7 @@ void PromptWidget::drawCaret()
 
   char c = buffer(_currentPos);
   s.fillRect(x, y, _kConsoleCharWidth, _kConsoleLineHeight, kTextColor);
-  s.drawChar(&_boss->instance().consoleFont(), c, x, y + 2, kBGColor);
+  s.drawChar(_boss->instance().consoleFont(), c, x, y + 2, kBGColor);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
