@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: RomListWidget.cxx 2356 2012-01-14 22:00:54Z stephena $
+// $Id: RomListWidget.cxx 2504 2012-05-27 19:27:55Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -65,9 +65,10 @@ RomListWidget::RomListWidget(GuiObject* boss, const GUI::Font& font,
   l.push_back("Save ROM", "saverom");
   l.push_back("Set PC", "setpc");
   l.push_back("RunTo PC", "runtopc");
-  l.push_back("---------------------", "");
+  l.push_back("-------------------------", "");
   l.push_back("Toggle PC addresses", "pcaddr");
   l.push_back("Toggle GFX binary/hex", "gfx");
+  l.push_back("Toggle address relocation", "relocate");
   l.push_back("Re-disassemble", "disasm");
   myMenu = new ContextMenu(this, font, l);
 
