@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TiaOutputWidget.cxx 2199 2011-01-01 16:04:32Z stephena $
+// $Id: TiaOutputWidget.cxx 2377 2012-01-29 22:54:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -50,6 +50,7 @@ TiaOutputWidget::TiaOutputWidget(GuiObject* boss, const GUI::Font& font,
   l.push_back("Fill to scanline", "scanline");
   l.push_back("Set breakpoint", "bp");
   l.push_back("Set zoom position", "zoom");
+  l.push_back("Toggle fixed debug colors", "fixed");
   myMenu = new ContextMenu(this, font, l);
 }
 
@@ -114,6 +115,10 @@ void TiaOutputWidget::handleCommand(CommandSender* sender, int cmd, int data, in
         if(myZoom)
           myZoom->setPos(myClickX, myClickY);
       }
+      else if(rmb == "fixed")
+      {
+        instance().console().tia().toggleFixedColors();
+      }
       break;
     }
   }
