@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TiaWidget.cxx 2260 2011-07-03 17:08:42Z stephena $
+// $Id: TiaWidget.cxx 2343 2012-01-08 16:55:10Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -45,64 +45,11 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& font,
   const int fontWidth  = font.getMaxCharWidth(),
             fontHeight = font.getFontHeight(),
             lineHeight = font.getLineHeight();
-  int xpos = 10, ypos = 10, lwidth = 4 * font.getMaxCharWidth();
+  int xpos = 10, ypos = 15 + lineHeight, lwidth = 4 * font.getMaxCharWidth();
   StaticTextWidget* t;
 
-  // Create a 16x1 grid holding byte values with labels
-  // Only do so if we have the vertical height
-  // TODO - at some point, viewing RAM may be removed, as I don't know
-  //        how useful it really is
-  if(h >= 400)
-  {
-    ypos += lineHeight;
-    myRamGrid = new DataGridWidget(boss, font, xpos + lwidth, ypos,
-                                   16, 1, 2, 8, kBASE_16);
-    myRamGrid->setEditable(false);
-    myRamGrid->setTarget(this);
-    myRamGrid->setID(kRamID);
-    addFocusWidget(myRamGrid);
-
-    t = new StaticTextWidget(boss, font, xpos, ypos + 2,
-                             lwidth-2, fontHeight,
-                             "00:", kTextAlignLeft);
-    for(int col = 0; col < 16; ++col)
-    {
-      t = new StaticTextWidget(boss, font, xpos + col*myRamGrid->colWidth() + lwidth + 7,
-                               ypos - lineHeight,
-                               fontWidth, fontHeight,
-                               instance().debugger().valueToString(col, kBASE_16_1),
-                               kTextAlignLeft);
-    }
-  
-    xpos = 20;  ypos += 2 * lineHeight;
-    t = new StaticTextWidget(boss, font, xpos, ypos, 6*fontWidth, fontHeight,
-                             "Label:", kTextAlignLeft);
-    xpos += 6*fontWidth + 5;
-    myLabel = new EditTextWidget(boss, font, xpos, ypos-2, 15*fontWidth,
-                                 lineHeight, "");
-    myLabel->setEditable(false);
-
-    xpos += 15*fontWidth + 20;
-    t = new StaticTextWidget(boss, font, xpos, ypos, 4*fontWidth, fontHeight,
-                             "Dec:", kTextAlignLeft);
-    xpos += 4*fontWidth + 5;
-    myDecValue = new EditTextWidget(boss, font, xpos, ypos-2, 4*fontWidth,
-                                    lineHeight, "");
-    myDecValue->setEditable(false);
-
-    xpos += 4*fontWidth + 20;
-    t = new StaticTextWidget(boss, font, xpos, ypos, 4*fontWidth, fontHeight,
-                             "Bin:", kTextAlignLeft);
-    xpos += 4*fontWidth + 5;
-    myBinValue = new EditTextWidget(boss, font, xpos, ypos-2, 9*fontWidth,
-                                    lineHeight, "");
-    myBinValue->setEditable(false);
-    ypos += lineHeight + 10;
-  }
-
   // Color registers
   const char* regNames[] = { "COLUP0:", "COLUP1:", "COLUPF:", "COLUBK:" };
-  xpos = 10;  ypos += lineHeight + 5;
   for(int row = 0; row < 4; ++row)
   {
     t = new StaticTextWidget(boss, font, xpos, ypos + row*lineHeight + 2,
@@ -593,14 +540,7 @@ TiaWidget::~TiaWidget()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TiaWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
 {
-  // We simply change the values in the DataGridWidget
-  // It will then send the 'kDGItemDataChangedCmd' signal to change the actual
-  // memory location
-  int addr, value;
-  string buf;
-
   Debugger& dbg = instance().debugger();
-  CartDebug& cart = dbg.cartDebug();
   TIADebug& tia = dbg.tiaDebug();
 
   switch(cmd)
@@ -745,23 +685,6 @@ void TiaWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       }
       break;
 
-    case kDGSelectionChangedCmd:
-      switch(id)
-      {
-        case kRamID:
-          addr  = myRamGrid->getSelectedAddr();
-          value = myRamGrid->getSelectedValue();
-
-          // We're using the read-addresses here
-          // Should we also add write-addresses, or remove this entirely?
-          myLabel->setEditString(cart.getLabel(addr, true));
-
-          myDecValue->setEditString(dbg.valueToString(value, kBASE_10));
-          myBinValue->setEditString(dbg.valueToString(value, kBASE_2));
-          break;
-      }
-      break;
-
     case kCheckActionCmd:
       switch(id)
       {
@@ -888,19 +811,6 @@ void TiaWidget::fillGrid()
   TiaState& state    = (TiaState&) tia.getState();
   TiaState& oldstate = (TiaState&) tia.getOldState();
 
-  // TIA RAM
-  if(myRamGrid)
-  {
-    alist.clear();  vlist.clear();  changed.clear();
-    for(unsigned int i = 0; i < 16; i++)
-    {
-      alist.push_back(i);
-      vlist.push_back(state.ram[i]);
-      changed.push_back(state.ram[i] != oldstate.ram[i]);
-    }
-    myRamGrid->setList(alist, vlist, changed);
-  }
-
   // Color registers
   alist.clear();  vlist.clear();  changed.clear();
   for(unsigned int i = 0; i < 4; i++)
