@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TiaZoomWidget.cxx 2199 2011-01-01 16:04:32Z stephena $
+// $Id: TiaZoomWidget.cxx 2335 2012-01-05 21:30:35Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -124,8 +124,8 @@ void TiaZoomWidget::recalc()
 
   // Only redraw when necessary
   int oldXoff = myXoff, oldYoff = myYoff;
-  myXoff = myXCenter - (myNumCols >> 1);
-  myYoff = myYCenter - (myNumRows >> 1);
+  myXoff = BSPF_max(0, myXCenter - (myNumCols >> 1));
+  myYoff = BSPF_max(0, myYCenter - (myNumRows >> 1));
   if(oldXoff != myXoff || oldYoff != myYoff)
   {
     setDirty(); draw();
