@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Cart3F.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: Cart3F.cxx 2499 2012-05-25 12:41:19Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -192,11 +192,11 @@ bool Cartridge3F::save(Serializer& out) const
   try
   {
     out.putString(name());
-    out.putInt(myCurrentBank);
+    out.putShort(myCurrentBank);
   }
-  catch(const char* msg)
+  catch(...)
   {
-    cerr << "ERROR: Cartridge3F::save" << endl << "  " << msg << endl;
+    cerr << "ERROR: Cartridge3F::save" << endl;
     return false;
   }
 
@@ -211,11 +211,11 @@ bool Cartridge3F::load(Serializer& in)
     if(in.getString() != name())
       return false;
 
-    myCurrentBank = (uInt16) in.getInt();
+    myCurrentBank = in.getShort();
   }
-  catch(const char* msg)
+  catch(...)
   {
-    cerr << "ERROR: Cartridge3F::load" << endl << "  " << msg << endl;
+    cerr << "ERROR: Cartridge3F::load" << endl;
     return false;
   }
 
