@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Cart4K.cxx 2325 2012-01-02 20:31:42Z stephena $
+// $Id: Cart4K.cxx 2499 2012-05-25 12:41:19Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -118,9 +118,9 @@ bool Cartridge4K::save(Serializer& out) const
   {
     out.putString(name());
   }
-  catch(const char* msg)
+  catch(...)
   {
-    cerr << "ERROR: Cartridge4K::save" << endl << "  " << msg << endl;
+    cerr << "ERROR: Cartridge4K::save" << endl;
     return false;
   }
 
@@ -135,9 +135,9 @@ bool Cartridge4K::load(Serializer& in)
     if(in.getString() != name())
       return false;
   }
-  catch(const char* msg)
+  catch(...)
   {
-    cerr << "ERROR: Cartridge4K::load" << endl << "  " << msg << endl;
+    cerr << "ERROR: Cartridge4K::load" << endl;
     return false;
   }
 
