@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartCM.cxx 2414 2012-03-15 15:22:57Z stephena $
+// $Id: CartCM.cxx 2499 2012-05-25 12:41:19Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -202,19 +202,14 @@ bool CartridgeCM::save(Serializer& out) const
   try
   {
     out.putString(name());
-    out.putInt(myCurrentBank);
+    out.putShort(myCurrentBank);
     out.putByte(myRamState);
     out.putByte(myColumn);
-
-    // The 2048 bytes of RAM
-    out.putInt(2048);
-    for(uInt32 i = 0; i < 2048; ++i)
-      out.putByte((char)myRAM[i]);
-
+    out.putByteArray(myRAM, 2048);
   }
-  catch(const char* msg)
+  catch(...)
   {
-    cerr << "ERROR: CartridgeCM::save" << endl << "  " << msg << endl;
+    cerr << "ERROR: CartridgeCM::save" << endl;
     return false;
   }
 
@@ -229,18 +224,14 @@ bool CartridgeCM::load(Serializer& in)
     if(in.getString() != name())
       return false;
 
-    myCurrentBank = (uInt16) in.getInt();
-    myRamState = (uInt8) in.getByte();
-    myColumn = (uInt8) in.getByte();
-
-    // The 2048 bytes of RAM
-    uInt32 limit = (uInt32) in.getInt();
-    for(uInt32 i = 0; i < limit; ++i)
-      myRAM[i] = (uInt8) in.getByte();
+    myCurrentBank = in.getShort();
+    myRamState = in.getByte();
+    myColumn = in.getByte();
+    in.getByteArray(myRAM, 2048);
   }
-  catch(const char* msg)
+  catch(...)
   {
-    cerr << "ERROR: CartridgeCM::load" << endl << "  " << msg << endl;
+    cerr << "ERROR: CartridgeCM::load" << endl;
     return false;
   }
 
