@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartEFSC.cxx 2325 2012-01-02 20:31:42Z stephena $
+// $Id: CartEFSC.cxx 2499 2012-05-25 12:41:19Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -208,11 +208,12 @@ bool CartridgeEFSC::save(Serializer& out) const
   try
   {
     out.putString(name());
-    out.putInt(myCurrentBank);
+    out.putShort(myCurrentBank);
+    out.putByteArray(myRAM, 128);
   }
-  catch(const char* msg)
+  catch(...)
   {
-    cerr << "ERROR: CartridgeEFSC::save" << endl << "  " << msg << endl;
+    cerr << "ERROR: CartridgeEFSC::save" << endl;
     return false;
   }
 
@@ -227,11 +228,12 @@ bool CartridgeEFSC::load(Serializer& in)
     if(in.getString() != name())
       return false;
 
-    myCurrentBank = (uInt16) in.getInt();
+    myCurrentBank = in.getShort();
+    in.getByteArray(myRAM, 128);
   }
-  catch(const char* msg)
+  catch(...)
   {
-    cerr << "ERROR: CartridgeEFSC::load" << endl << "  " << msg << endl;
+    cerr << "ERROR: CartridgeEFSC::load" << endl;
     return false;
   }
 
