@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartF4.cxx 2325 2012-01-02 20:31:42Z stephena $
+// $Id: CartF4.cxx 2499 2012-05-25 12:41:19Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -151,11 +151,11 @@ bool CartridgeF4::save(Serializer& out) const
   try
   {
     out.putString(name());
-    out.putInt(myCurrentBank);
+    out.putShort(myCurrentBank);
   }
-  catch(const char* msg)
+  catch(...)
   {
-    cerr << "ERROR: CartridgeF4::save" << endl << "  " << msg << endl;
+    cerr << "ERROR: CartridgeF4::save" << endl;
     return false;
   }
 
@@ -170,11 +170,11 @@ bool CartridgeF4::load(Serializer& in)
     if(in.getString() != name())
       return false;
 
-    myCurrentBank = (uInt16)in.getInt();
+    myCurrentBank = in.getShort();
   }
-  catch(const char* msg)
+  catch(...)
   {
-    cerr << "ERROR: CartridgeF4::load" << endl << "  " << msg << endl;
+    cerr << "ERROR: CartridgeF4::load" << endl;
     return false;
   }
 
