@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartF4SC.cxx 2325 2012-01-02 20:31:42Z stephena $
+// $Id: CartF4SC.cxx 2499 2012-05-25 12:41:19Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -211,16 +211,12 @@ bool CartridgeF4SC::save(Serializer& out) const
   try
   {
     out.putString(name());
-    out.putInt(myCurrentBank);
-
-    // The 128 bytes of RAM
-    out.putInt(128);
-    for(uInt32 i = 0; i < 128; ++i)
-      out.putByte((char)myRAM[i]);
+    out.putShort(myCurrentBank);
+    out.putByteArray(myRAM, 128);
   }
-  catch(const char* msg)
+  catch(...)
   {
-    cerr << "ERROR: CartridgeF4SC::save" << endl << "  " << msg << endl;
+    cerr << "ERROR: CartridgeF4SC::save" << endl;
     return false;
   }
 
@@ -235,15 +231,12 @@ bool CartridgeF4SC::load(Serializer& in)
     if(in.getString() != name())
       return false;
 
-    myCurrentBank = (uInt16) in.getInt();
-
-    uInt32 limit = (uInt32) in.getInt();
-    for(uInt32 i = 0; i < limit; ++i)
-      myRAM[i] = (uInt8) in.getByte();
+    myCurrentBank = in.getShort();
+    in.getByteArray(myRAM, 128);
   }
-  catch(const char* msg)
+  catch(...)
   {
-    cerr << "ERROR: CartridgeF4SC::load" << endl << "  " << msg << endl;
+    cerr << "ERROR: CartridgeF4SC::load" << endl;
     return false;
   }
 
